"""Fix broken (non-physical) inertias in SDFormat/URDF, by writing a revised
file.

The common data structure being repaired in both formats is the content of the
`<inertial>` tag, which contains the mass, the inertia tensor, and also defines
a central inertia frame in which the tensor is expressed.

By definitions used in both URDF and SDFormat documentation, the relevant
frames are the link origin frame (L) and the central inertia frame (C). Drake
software often uses the terminology "body frame" (B), which is equivalent to L.
Therefore, in Drake monogram notation, the inertia tensor is expressed in B,
and the `<inertial>` tag includes the pose of C relative to B, X_BC.

"""

from __future__ import annotations

import abc
from collections.abc import Callable
from dataclasses import dataclass, field
from pathlib import Path
import xml.parsers.expat as expat

import numpy as np

from pydrake.geometry import Role, SceneGraph
from pydrake.multibody.parsing import Parser
from pydrake.multibody.plant import MultibodyPlant
from pydrake.multibody.tree import (
    BodyIndex,
    CalcSpatialInertia,
    RigidBody,
    SpatialInertia,
    UnitInertia,
)


@dataclass
class SourceLocation:
    """The location of an XML syntax item as reported by expat parsing."""

    index: int
    "0-based byte index from start of input stream."

    line: int
    "1-based line number from start of input stream."


@dataclass
class ElementFacts:
    """Facts about an XML element as collecting by expat parsing."""

    prior_data: str
    "Character data immediately preceding the element text."

    name: str
    "The name of the XML element."

    attributes: dict
    "The attributes of the XML element."

    depth: int
    "The nesting depth of the XML element; 0 for the root element."

    start: SourceLocation
    "The location of the start of the XML element text."

    end: SourceLocation = None
    """The location of the end of the XML element text; may require
    adjustments, depending on the input syntax used (optional).
    """

    parent: ElementFacts = None
    "Facts for the parent of this element (optional)."

    children: list[ElementFacts] = field(default_factory=list)
    "Facts about relevant children of this element (optional)."

    parent_model: ElementFacts = None
    """Facts for the enclosing model of this model element (optional).
    The definition of a model element is determined by each file format;
    see FormatDriver.is_model_element().
    """


# This function has an aggressively short name for embedding into f-strings.
def gfmt(x: float) -> str:
    """Converts a floating point number to a string, using some rounding to
    avoid platform differences in results.
    """
    return f"{x:.5g}"


class FormatDriver(metaclass=abc.ABCMeta):
    """This interface encapsulates the differences between URDF and SDFormat
    processing, for purposes of the implementation of XmlInertiaMapper.
    """

    @abc.abstractmethod
    def is_model_element(self, name: str) -> bool:
        """Returns True if the element directly encloses link elements.
        Args:
            name: the name of the XML element
        """
        raise NotImplementedError

    @abc.abstractmethod
    def associate_plant_models(
        self, models: list, links: list, plant: MultibodyPlant
    ) -> dict:
        """Returns a mapping of body_index to inertial ElementFacts.

        Args:
            models: a list of ElementFacts for model elements
            links: a list of ElementFacts for link elements
            plant: a MultibodyPlant containing the parsed models
        """
        raise NotImplementedError

    @abc.abstractmethod
    def format_inertia(
        self,
        input_bytes: bytes,
        inertial_facts: ElementFacts,
        pose: list,
        mass: float,
        moments: list,
        products: list,
    ) -> bytes:
        """Returns formatted XML bytes for the repaired inertial tag.

        See above for definitions of the frames involved.

        Args:
            input_bytes: full contents of the input file
            inertial_facts: facts about the element to reformat
            pose: Pose of the central inertia frame (C) relative to the body
                  frame (B), as a 6-element list: [x, y, z, r, p, y].
            mass: the mass of the body (kg).
            moments: the principle moments of inertia, expressed in C --
                     [ixx, iyy, izz].
            products: the products of inertia, expressed in C --
                      [ixy, ixz, iyz].
        """
        raise NotImplementedError


def is_synth_element(el: ElementFacts) -> bool:
    """Returns True if the ElementFacts is synthetic.

    Synthetic elements are placeholders for later expansion into output XML.
    """
    return el.end.index == el.start.index


def make_synth_element(parent: ElementFacts, name: str) -> ElementFacts:
    """Makes a new synthetic child ElementFacts for a given parent
    ElementFacts. The parent must already have at least one child.

    The returned synthetic element will be used for later expansion into the
    output XML.

    Args:
        parent: the ElementFacts that will get a new synthetic child
        name: the XML element name of the new child

    """
    assert parent.children, parent
    kid0 = parent.children[0]
    synth_el = ElementFacts(kid0.prior_data, name, {}, kid0.depth, kid0.start)
    synth_el.end = synth_el.start
    return synth_el


def find_inertial_facts_for_link(link: ElementFacts) -> ElementFacts:
    """Fetches or synthesizes a child inertial ElementFacts for a link
    ElementFacts.
    """
    inertial_kids = [x for x in link.children if x.name == "inertial"]
    if inertial_kids:
        return inertial_kids[0]
    return make_synth_element(link, "inertial")


def maybe_synthesize_children(parent: ElementFacts, names: list[str]):
    """Given a parent ElementFacts and a list of required child elements,
    synthesizes ElementFacts for any children that are missing. The parent must
    already have at least one child.

    The new synthetic elements will be used for later expansion into the output
    XML. Note that synthetic elements are inserted at the beginning of the list
    of child elements (for formatting convenience), so they will appear in the
    formatted output in the reverse order of that given by the input list of
    names.

    Args:
        parent: the ElementFacts that will get new synthetic children
        names: a list of XML element names to insert if missing
    """
    assert parent.children, parent
    kid_names = [x.name for x in parent.children]
    for needed_name in names:
        if needed_name in kid_names:
            continue
        parent.children.insert(0, make_synth_element(parent, needed_name))


def adjusted_element_end_index(input_bytes: bytes, facts: ElementFacts) -> int:
    """Returns the index of the end of all of the text (including the element
    closure), when given an ElementFacts produced by the expat parse.

    Args:
        input_bytes: full contents of the input file
        facts: an ElementFacts structure
    """
    if is_synth_element(facts):
        # Empty, synthetic, "pseudo" element.
        return facts.end.index

    if input_bytes[facts.end.index :].startswith(
        f"</{facts.name}".encode("utf-8")
    ):
        # Typical case:
        # `<inertial>...</inertial>`.
        #               ^                # Raw index points here.
        #                          ^     # Adjusted index points here.
        return facts.end.index + input_bytes[facts.end.index :].find(b">") + 1

    # `<inertial/>` corner case.
    #             ^     # Raw index points here; good to go
    return facts.end.index


def make_format_helpers(
    facts: ElementFacts,
) -> (Callable[[int], str], Callable[[], str]):
    """Returns two callable functions that help format white space for output
    XML elements.

    The returned callables are intended to be convenient to invoke in Python
    f-strings; callers typically assign them as `d` and `coda`.

    d(int) -> str
        indents n levels in addition to the current depth of the element
        described by `facts`.

    coda() -> str
        provides appropriate trailing white space to maintain typical
        formatting:
            * newline plus d(0) if `facts` is synthetic,
            * otherwise, empty string.

    Args:
        facts: an ElementFacts structure

    Returns:
        d: the indent function described above
        coda: the trailing space function described above
    """
    spaces = len(facts.prior_data)
    indentation = spaces // facts.depth

    # Build an element-specific indenter.
    depth = facts.depth
    indent = " " * indentation

    def d(more_depth):
        return indent * (depth + more_depth)

    synth = facts.end.index == facts.start.index

    def coda():
        return "\n" + d(0) if synth else ""

    return (d, coda)


class UrdfDriver(FormatDriver):
    """Format driver for URDF files."""

    def is_model_element(self, name: str) -> bool:
        return name == "robot"

    def _is_element_ignored(self, facts: ElementFacts) -> bool:
        # TODO(rpoyner_tri): The 'drake_ignore' attribute is regrettable legacy
        # cruft that should be removed when the associated URDF parser cruft is
        # removed.
        return (facts.attributes.get("drake_ignore") == "true") or (
            facts.attributes.get("name") == "world"
        )

    def associate_plant_models(
        self, models: list, links: list, plant: MultibodyPlant
    ) -> dict:
        # This assertion is weakened for files that use 'drake_ignore'.
        assert len(links) >= plant.num_bodies() - 1, (links, plant.num_bodies())
        mapping = {}
        serial_number = 0
        for link in links:
            if self._is_element_ignored(link):
                continue
            serial_number += 1
            if not link.children:
                # Without inertia (mass) or geometry, there is nothing to do.
                continue
            link_name = link.attributes.get("name")
            assert serial_number < plant.num_bodies()
            bix = BodyIndex(serial_number)
            mapping[bix] = find_inertial_facts_for_link(link)
            assert plant.get_body(bix).name() == link_name, (
                plant.get_body(bix).name(),
                bix,
                link_name,
            )
        return mapping

    def format_inertia(
        self,
        input_bytes: bytes,
        inertial_facts: ElementFacts,
        pose: list,
        mass: float,
        moments: list,
        products: list,
    ) -> bytes:
        end = adjusted_element_end_index(input_bytes, inertial_facts)
        inertia_output = f"""\
<inertia ixx="{gfmt(moments[0])}" ixy="{gfmt(products[0])}"\
 ixz="{gfmt(products[1])}" iyy="{gfmt(moments[1])}"\
 iyz="{gfmt(products[2])}" izz="{gfmt(moments[2])}"/>"""
        xyz_output = " ".join([gfmt(x) for x in pose[:3]])
        rpy_output = " ".join([gfmt(x) for x in pose[3:]])
        # * self-closing or no inertial tag: pave it all.
        if end == inertial_facts.end.index:
            d, coda = make_format_helpers(inertial_facts)
            return f"""\
<inertial>
{d(1)}{inertia_output}
{d(1)}<mass value="{gfmt(mass)}"/>
{d(1)}<origin rpy="{rpy_output}" xyz="{xyz_output}"/>
{d(0)}</inertial>{coda()}""".encode("utf-8")

        maybe_synthesize_children(inertial_facts, ["origin", "mass", "inertia"])
        # Now we get to slice up the inertial element, and only rewrite the
        # parts that we must.
        output = bytes()
        index = inertial_facts.start.index
        for facts in inertial_facts.children:
            output += input_bytes[index : facts.start.index]
            d, coda = make_format_helpers(facts)
            if facts.name == "inertia":
                output += f"{inertia_output}{coda()}".encode("utf-8")
            if facts.name == "mass":
                output += f'<mass value="{gfmt(mass)}"/>{coda()}'.encode(
                    "utf-8"
                )
            if facts.name == "origin":
                output += f"""\
<origin rpy="{rpy_output}" xyz="{xyz_output}"/>{coda()}""".encode("utf-8")
            index = adjusted_element_end_index(input_bytes, facts)
        output += input_bytes[index:end]
        return output


class SdformatDriver(FormatDriver):
    """Format driver for SDFormat files."""

    def is_model_element(self, name: str) -> bool:
        return name == "model"

    def associate_plant_models(
        self, models: list, links: list, plant: MultibodyPlant
    ) -> dict:
        # Because SDFormat has both nested models and inclusions, we will have
        # to rummage around in the plant, finding body indices by using name
        # strings.
        mapping = {}
        for link in links:
            if not link.children:
                continue
            model = link.parent
            enclosing_models = [model]
            while model.parent_model:
                model = model.parent_model
                enclosing_models.append(model)
            enclosing_models.reverse()
            model_name = "::".join(
                [m.attributes["name"] for m in enclosing_models]
            )
            model_instance = plant.GetModelInstanceByName(model_name)
            body = plant.GetBodyByName(link.attributes["name"], model_instance)
            bix = body.index()
            mapping[bix] = find_inertial_facts_for_link(link)
            assert plant.get_body(bix).name() == link.attributes.get("name"), (
                plant.get_body(bix).name(),
                bix,
                link.attributes.get("name"),
            )
        return mapping

    def format_inertia(
        self,
        input_bytes: bytes,
        inertial_facts: ElementFacts,
        pose: list,
        mass: float,
        moments: list,
        products: list,
    ) -> bytes:
        end = adjusted_element_end_index(input_bytes, inertial_facts)
        pose_output = " ".join([gfmt(x) for x in pose])
        # * self-closing or no inertial tag: pave it all.
        if end == inertial_facts.end.index:
            d, coda = make_format_helpers(inertial_facts)
            return f"""\
<inertial>
{d(1)}<inertia>
{d(2)}<ixx>{gfmt(moments[0])}</ixx>
{d(2)}<ixy>{gfmt(products[0])}</ixy>
{d(2)}<ixz>{gfmt(products[1])}</ixz>
{d(2)}<iyy>{gfmt(moments[1])}</iyy>
{d(2)}<iyz>{gfmt(products[2])}</iyz>
{d(2)}<izz>{gfmt(moments[2])}</izz>
{d(1)}</inertia>
{d(1)}<mass>{gfmt(mass)}</mass>
{d(1)}<pose>{pose_output}</pose>
{d(0)}</inertial>{coda()}""".encode("utf-8")

        maybe_synthesize_children(inertial_facts, ["pose", "mass", "inertia"])
        # Now we get to slice up the inertial element, and only rewrite the
        # parts that we must.
        output = bytes()
        index = inertial_facts.start.index
        for facts in inertial_facts.children:
            output += input_bytes[index : facts.start.index]
            d, coda = make_format_helpers(facts)
            if facts.name == "inertia":
                output += f"""\
<inertia>
{d(1)}<ixx>{gfmt(moments[0])}</ixx>
{d(1)}<ixy>{gfmt(products[0])}</ixy>
{d(1)}<ixz>{gfmt(products[1])}</ixz>
{d(1)}<iyy>{gfmt(moments[1])}</iyy>
{d(1)}<iyz>{gfmt(products[2])}</iyz>
{d(1)}<izz>{gfmt(moments[2])}</izz>
{d(0)}</inertia>{coda()}""".encode("utf-8")
            if facts.name == "mass":
                output += f"<mass>{gfmt(mass)}</mass>{coda()}".encode("utf-8")
            if facts.name == "pose":
                output += f"<pose>{pose_output}</pose>{coda()}".encode("utf-8")
            index = adjusted_element_end_index(input_bytes, facts)
        end = adjusted_element_end_index(input_bytes, inertial_facts)
        output += input_bytes[index:end]
        return output


class XmlInertiaMapper:
    """Handles the parsing, indexing, and output generation details for editing
    of inertias in some formats of XML robot model files.
    """

    def __init__(self, input_text: str):
        """Initialize an XmlInertiaMapper.

        Args:
            input_text: the full contents of the input file
        """
        self._input_text = input_text
        self._input_bytes = input_text.encode("utf-8")

        # Configure the parser.
        self._parser = expat.ParserCreate()
        self._parser.StartElementHandler = self._handle_start_element
        self._parser.EndElementHandler = self._handle_end_element
        self._parser.CharacterDataHandler = self._handle_char_data

        # Declare some state to remember during parsing.
        self._depth = 0
        self._element_stack = []
        self._models = []
        self._model_stack = []
        self._links = []
        self._format_driver = None
        self._last_char_data = ""

        # Eventually build a mapping from body_index to inertial_facts.
        self._mapping = {}

    def _make_location(self) -> SourceLocation:
        return SourceLocation(
            self._parser.CurrentByteIndex, self._parser.CurrentLineNumber
        )

    def _make_element_facts(self, name: str, attributes: dict) -> ElementFacts:
        return ElementFacts(
            self._last_char_data,
            name,
            attributes,
            self._depth,
            self._make_location(),
        )

    def _handle_char_data(self, data: str):
        self._last_char_data = data

    def _handle_start_element(self, name: str, attributes: dict):
        if not self._format_driver and self._depth == 0:
            if name == "robot":
                self._format_driver = UrdfDriver()
            elif name == "sdf":
                self._format_driver = SdformatDriver()
            else:
                raise RuntimeError("unknown file format!")

        element = self._make_element_facts(name, attributes)
        if self._element_stack:
            element.parent = self._element_stack[-1]
            element.parent.children.append(element)

        if self._format_driver.is_model_element(name):
            if self._model_stack:
                element.parent_model = self._model_stack[-1]
            self._models.append(element)
            self._model_stack.append(element)

        if name == "link":
            self._links.append(element)

        self._depth += 1
        self._element_stack.append(element)

    def _handle_end_element(self, name: str):
        self._element_stack[-1].end = self._make_location()
        self._element_stack.pop()
        self._depth -= 1

        if self._format_driver.is_model_element(name):
            self._model_stack.pop()

    def parse(self):
        """Execute the parsing of the XML text."""
        self._parser.Parse(self._input_text)

    def build_plant_models_association(self, plant: MultibodyPlant):
        """Matches body indices to inertial elements in the input text.

        Args:
            plant: a MultibodyPlant containing the parsed models
        """
        assert self._format_driver is not None
        self._mapping = self._format_driver.associate_plant_models(
            self._models, self._links, plant
        )

    def mapping(self) -> dict:
        """Returns a mapping from body indices to inertial element facts."""
        return self._mapping

    def build_output(self, new_inertias_mapping: dict) -> str:
        """Builds and returns the complete output text.

        Args:
            new_inertias_mapping: a dictionary whose keys are BodyIndex values,
               and whose values are a tuple of the body's spatial inertia and
               6D inertial pose as [x, y, z, r, p, y].
        """
        output = bytes()
        input_bytes = self._input_bytes
        input_bytes_index = 0
        for body_index, new_inertia in sorted(new_inertias_mapping.items()):
            inertial_facts = self._mapping[body_index]
            output += input_bytes[
                input_bytes_index : inertial_facts.start.index
            ]
            output += self._format_inertia(inertial_facts, *new_inertia)
            input_bytes_index = adjusted_element_end_index(
                input_bytes, inertial_facts
            )
        output += input_bytes[input_bytes_index:]
        return output.decode("utf-8")

    def _format_inertia(
        self,
        inertial_facts: ElementFacts,
        spatial_inertia: SpatialInertia,
        pose: list,
    ) -> str:
        # Extract the mass properties.
        mass = spatial_inertia.get_mass()
        rot = spatial_inertia.CalcRotationalInertia()
        mom = rot.get_moments()
        prod = rot.get_products()

        return self._format_driver.format_inertia(
            self._input_bytes, inertial_facts, pose, mass, mom, prod
        )


# Roles avialable for ordering.
GEOM_INERTIA_ROLE_AVAILABLE = [
    Role.kProximity,
    Role.kIllustration,
    Role.kPerception,
]
# The default ordering happens to be the same as the available roles.
GEOM_INERTIA_ROLE_ORDER_DEFAULT = GEOM_INERTIA_ROLE_AVAILABLE


class InertiaProcessor:
    """Handles selection, repair, and replacement of inertial properties,
    in model files pre-processed by drake parsing and XmlInertiaMapper.
    """

    def __init__(
        self,
        plant: MultibodyPlant,
        scene_graph: SceneGraph,
        mapper: XmlInertiaMapper,
        geom_inertia_role_order: list[Role],
    ):
        self._plant = plant
        self._scene_graph = scene_graph
        self._mapper = mapper
        self._geom_inertia_role_order = geom_inertia_role_order

    def _maybe_fix_inertia(self, body_index: BodyIndex) -> [
        SpatialInertia,
        list,
    ] | None:
        assert int(body_index) < self._plant.num_bodies(), (
            body_index,
            self._plant.num_bodies(),
        )
        body = self._plant.get_body(body_index)
        if not isinstance(body, RigidBody):
            # Only rigid bodies have constant inertia, for which model file
            # fixups make sense.
            return
        frame_id = self._plant.GetBodyFrameIdOrThrow(body_index)
        old_inertia = body.default_spatial_inertia()
        inspector = self._scene_graph.model_inspector()
        for role in self._geom_inertia_role_order:
            geoms = inspector.GetGeometries(frame_id, role)
            if geoms:
                break
        if not geoms:
            # No geometry to fix inertia from.
            return

        # Collect some density==1 inertias for all geometries. The relevant
        # frames named below are:
        # * B: the body frame
        # * G: the frame of a single attached geometry
        # * Bcm: the body frame, translated to the center of mass, but not
        #   rotated.
        M_BBo_B_one = SpatialInertia(0, np.zeros(3), UnitInertia(0, 0, 0))
        for geom in geoms:
            M_GG_G_one = CalcSpatialInertia(inspector.GetShape(geom), 1.0)
            X_BG = inspector.GetPoseInFrame(geom)
            M_GBo_B_one = M_GG_G_one.ReExpress(X_BG.rotation()).Shift(
                -X_BG.translation()
            )
            M_BBo_B_one += M_GBo_B_one

            # Rebuild the final inertia using the mass found in the input.
            mass = old_inertia.get_mass()
            M_BBo_B = SpatialInertia(
                mass, M_BBo_B_one.get_com(), M_BBo_B_one.get_unit_inertia()
            )

        # TODO(rpoyner-tri): rotate to zero out products of inertia, and return
        # both inertia matrix and pose.
        M_BBcm_B = M_BBo_B.Shift(M_BBo_B.get_com())
        pose = [x for x in M_BBo_B.get_com()] + [0.0, 0.0, 0.0]

        return M_BBcm_B, pose

    def process(self) -> str:
        """Returns a new model file text, with selected inertial properties
        replaced by new ones calculated from supplied geometry.
        """
        mapping = self._mapper.mapping()
        new_inertias_mapping = {}
        for body_index in sorted(mapping.keys()):
            maybe_inertia = self._maybe_fix_inertia(body_index)
            if maybe_inertia is not None:
                new_inertias_mapping[body_index] = maybe_inertia
        return self._mapper.build_output(new_inertias_mapping)


def fix_inertia_from_string(
    input_text: str,
    file_type: str,
    geom_inertia_role_order: list[Role] = GEOM_INERTIA_ROLE_ORDER_DEFAULT,
) -> str:
    """A pure string-processing wrapper for the tool, for testing."""
    # Parse with drake to build mbp and confirm sanity.
    plant = MultibodyPlant(time_step=0.0)
    scene_graph = SceneGraph()
    plant.RegisterAsSourceForSceneGraph(scene_graph)

    parser = Parser(plant)
    parser.package_map().PopulateFromRosPackagePath()

    parser.AddModelsFromString(input_text, file_type)

    # Parse with expat to build index.
    mapper = XmlInertiaMapper(input_text)
    mapper.parse()
    mapper.build_plant_models_association(plant)

    # Fix indicated inertias.
    processor = InertiaProcessor(
        plant, scene_graph, mapper, geom_inertia_role_order
    )
    return processor.process()


_role_to_str = {
    Role.kProximity: "proximity",
    Role.kIllustration: "illustration",
    Role.kPerception: "perception",
}
_str_to_role = {v: k for k, v in _role_to_str.items()}


def role_list_to_str_list(roles):
    return [_role_to_str[x] for x in roles]


def str_list_to_role_list(names):
    return [_str_to_role[x] for x in names]


def _is_unique(x):
    return len(set(x)) == len(x)


class InertiaFixer:
    """Fixes specified inertias in a URDF or SDFormat input file, writing the
    new file contents to one of: stdout, a new file, or the original file.
    """

    def __init__(
        self,
        *,
        input_file: Path,
        output_file: Path = None,
        in_place: bool = False,
        geom_inertia_role_order: list[Role] = GEOM_INERTIA_ROLE_ORDER_DEFAULT,
    ):
        """Initialize an InertiaFixer.

        Args:
            input_file: the file to fix.
            output_file: a file to be created or overwritten with the new file
                         contents. If None, the new contents will be written to
                         stdout.
            in_place: if True, write output back to the input file.
            geom_inertia_role_order: Specifies what order of geometries to try
                                     and infer geometry from for each body.
        """
        self.input_file = input_file
        self.output_file = output_file
        self.in_place = in_place
        if not _is_unique(geom_inertia_role_order):
            raise RuntimeError(
                "geom_inertia_role_order must have unique elements"
            )
        self.geom_inertia_role_order = geom_inertia_role_order

    def fix_inertia(self):
        """Executes the inertia fixing processing and write the output as
        specified by the initialization arguments.
        """
        # Parse with drake to build mbp and confirm sanity.
        plant = MultibodyPlant(time_step=0.0)
        scene_graph = SceneGraph()
        plant.RegisterAsSourceForSceneGraph(scene_graph)

        parser = Parser(plant)
        parser.package_map().PopulateFromRosPackagePath()

        # Read from the disk file here, to get more lenient processing of URIs,
        # better error messages, etc.
        parser.AddModels(str(self.input_file))

        # Slurp input file for indexing and editing.
        with open(self.input_file, encoding="utf-8") as fo:
            input_text = fo.read()

        # Parse with expat to build index.
        mapper = XmlInertiaMapper(input_text)
        mapper.parse()
        mapper.build_plant_models_association(plant)

        # Fix indicated inertias.
        processor = InertiaProcessor(
            plant, scene_graph, mapper, self.geom_inertia_role_order
        )
        output_text = processor.process()

        # Write output.
        if self.output_file:
            output_file = self.output_file
        else:
            output_file = "/dev/stdout"
        if self.in_place:
            output_file = self.input_file
        with open(output_file, "w", encoding="utf-8") as fo:
            fo.write(output_text)



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

import drake

class lcmt_call_python(object):
    """
    Message to support asynchronous remote procedure calls to a Python
    client. For historical reasons, interface is modeled on mexCallMATLAB
    (see https://www.mathworks.com/help/matlab/apiref/mexcallmatlab.html) but
    with output arguments assigned in the remote client workspace due to the
    asynchronous nature of the protocol.
    """

    __slots__ = ["function_name", "lhs", "num_rhs", "rhs"]

    __typenames__ = ["string", "int64_t", "int32_t", "drake.lcmt_call_python_data"]

    __dimensions__ = [None, None, None, ["num_rhs"]]

    def __init__(self):
        self.function_name = ""
        """
        Any expression that resolves to a callable Python object.
        LCM Type: string
        """

        self.lhs = 0
        """
        Unique id for variable held in the client workspace.
        LCM Type: int64_t
        """

        self.num_rhs = 0
        """
        Input argument data.
        LCM Type: int32_t
        """

        self.rhs = []
        """ LCM Type: drake.lcmt_call_python_data[num_rhs] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_call_python._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        __function_name_encoded = self.function_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__function_name_encoded)+1))
        buf.write(__function_name_encoded)
        buf.write(b"\0")
        buf.write(struct.pack(">qi", self.lhs, self.num_rhs))
        for i0 in range(self.num_rhs):
            assert self.rhs[i0]._get_packed_fingerprint() == drake.lcmt_call_python_data._get_packed_fingerprint()
            self.rhs[i0]._encode_one(buf)

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_call_python._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_call_python._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_call_python()
        __function_name_len = struct.unpack('>I', buf.read(4))[0]
        self.function_name = buf.read(__function_name_len)[:-1].decode('utf-8', 'replace')
        self.lhs, self.num_rhs = struct.unpack(">qi", buf.read(12))
        self.rhs = []
        for i0 in range(self.num_rhs):
            self.rhs.append(drake.lcmt_call_python_data._decode_one(buf))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_call_python in parents: return 0
        newparents = parents + [lcmt_call_python]
        tmphash = (0x30b0ddcf5c354914+ drake.lcmt_call_python_data._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_call_python._packed_fingerprint is None:
            lcmt_call_python._packed_fingerprint = struct.pack(">Q", lcmt_call_python._get_hash_recursive([]))
        return lcmt_call_python._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_call_python._get_packed_fingerprint())[0]




"""
This example demonstrates using Monte Carlo verification to analyze task
performance, for the task of stabilizing an inverted torque-limited pendulum
with LQR.
"""

import argparse

import numpy as np

from pydrake.common import RandomGenerator, configure_logging
from pydrake.multibody.parsing import Parser
from pydrake.multibody.plant import MultibodyPlant
from pydrake.symbolic import Variable
from pydrake.systems.analysis import MonteCarloSimulation, Simulator
from pydrake.systems.controllers import LinearQuadraticRegulator
from pydrake.systems.framework import DiagramBuilder
from pydrake.systems.primitives import Saturation


def main():
    configure_logging()
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "--num_samples",
        type=int,
        default=50,
        help="Number of Monte Carlo samples to use to estimate performance.",
    )
    parser.add_argument(
        "--torque_limit",
        type=float,
        default=2.0,
        help="Torque limit of the pendulum.",
    )
    args = parser.parse_args()
    if args.torque_limit < 0:
        raise ValueError("Please supply a nonnegative torque limit.")

    # Assemble the Pendulum plant.
    builder = DiagramBuilder()
    pendulum = builder.AddSystem(MultibodyPlant(0.0))
    Parser(builder, plant=pendulum).AddModelsFromUrl(
        url="package://drake/examples/pendulum/Pendulum.urdf"
    )
    pendulum.Finalize()

    # Set the pendulum to start at uniformly random
    # positions (but always zero velocity).
    elbow = pendulum.GetMutableJointByName("theta")
    upright_theta = np.pi
    theta_expression = (
        Variable(name="theta", type=Variable.Type.RANDOM_UNIFORM) * 2.0 * np.pi
    )
    elbow.set_random_angle_distribution(theta_expression)

    # Set up LQR, with high position gains to try to ensure the
    # ROA is close to the theoretical torque-limited limit.
    Q = np.diag([100.0, 1.0])
    R = np.identity(1) * 0.01
    linearize_context = pendulum.CreateDefaultContext()
    linearize_context.SetContinuousState(np.array([upright_theta, 0.0]))
    actuation_port = pendulum.get_actuation_input_port()
    actuation_port.FixValue(linearize_context, 0)
    controller = builder.AddSystem(
        LinearQuadraticRegulator(
            pendulum,
            linearize_context,
            Q,
            R,
            np.zeros(0),
            actuation_port.get_index(),
        )
    )

    # Apply the torque limit.
    torque_limit = args.torque_limit
    torque_limiter = builder.AddSystem(
        Saturation(
            min_value=np.array([-torque_limit]),
            max_value=np.array([torque_limit]),
        )
    )

    builder.Connect(
        controller.get_output_port(0), torque_limiter.get_input_port(0)
    )
    builder.Connect(
        torque_limiter.get_output_port(0), pendulum.get_actuation_input_port()
    )
    builder.Connect(
        pendulum.get_state_output_port(), controller.get_input_port(0)
    )
    diagram = builder.Build()

    # Perform the Monte Carlo simulation.
    def make_simulator(generator):
        """Create a simulator for the system
        using the given generator."""
        simulator = Simulator(diagram)
        simulator.set_target_realtime_rate(0)
        simulator.Initialize()
        return simulator

    def calc_wrapped_error(system, context):
        """Given a context from the end of the simulation,
        calculate an error -- which for this stabilizing
        task is the distance from the
        fixed point."""
        state = diagram.GetSubsystemContext(
            pendulum, context
        ).get_continuous_state_vector()
        error = state.GetAtIndex(0) - upright_theta
        # Wrap error to [-pi, pi].
        return (error + np.pi) % (2 * np.pi) - np.pi

    num_samples = args.num_samples
    results = MonteCarloSimulation(
        make_simulator=make_simulator,
        output=calc_wrapped_error,
        final_time=1.0,
        num_samples=num_samples,
        generator=RandomGenerator(),
    )

    # Compute results.
    # The "success" region is fairly large since some "stabilized" trials
    # may still be oscillating around the fixed point. Failed examples are
    # consistently much farther from the fixed point than this.
    binary_results = np.array([abs(res.output) < 0.1 for res in results])
    passing_ratio = float(sum(binary_results)) / len(results)
    # 95% confidence interval for the passing ratio.
    passing_ratio_var = 1.96 * np.sqrt(
        passing_ratio * (1.0 - passing_ratio) / len(results)
    )

    print(
        "Monte-Carlo estimated performance across %d samples: "
        "%.2f%% +/- %0.2f%%"
        % (num_samples, passing_ratio * 100, passing_ratio_var * 100)
    )

    # Analytically compute the best possible ROA, for comparison, but
    # calculating where the torque needed to lift the pendulum exceeds
    # the torque limit.
    arm_radius = 0.5
    arm_mass = 1.0
    # torque = r x f = r * (m * 9.81 * sin(theta))
    # theta = asin(torque / (r * m))
    if torque_limit <= (arm_radius * arm_mass * 9.81):
        roa_half_width = np.arcsin(
            torque_limit / (arm_radius * arm_mass * 9.81)
        )
    else:
        roa_half_width = np.pi

    roa_as_fraction_of_state_space = roa_half_width / np.pi
    print(
        "Max possible ROA = %0.2f%% of state space, which should"
        " match with the above estimate."
        % (100 * roa_as_fraction_of_state_space)
    )


if __name__ == "__main__":
    main()



import numpy
import pydrake.autodiffutils
import pydrake.common.eigen_geometry
import pydrake.geometry
import pydrake.geometry.optimization
import pydrake.math
import pydrake.multibody.math
import pydrake.multibody.optimization
import pydrake.multibody.plant
import pydrake.multibody.tree
import pydrake.perception
import pydrake.solvers
import pydrake.symbolic
import pydrake.systems.framework
import pydrake.systems.sensors
from pydrake.common.cpp_param import List as List
from pydrake.common.cpp_template import Value as Value
from typing import Any, Callable, ClassVar, overload

class AbstractValue:
    Make: ClassVar[Callable] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def Clone(self) -> AbstractValue: ...
    def SetFrom(self, arg0: AbstractValue) -> None: ...
    def get_mutable_value(self, *args, **kwargs) -> None: ...
    def get_value(self, *args, **kwargs) -> None: ...
    def set_value(self, *args, **kwargs) -> None: ...
    def __copy__(self) -> AbstractValue: ...
    def __deepcopy__(self, arg0: dict) -> AbstractValue: ...

class Value𝓣AbstractParameterIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.AbstractParameterIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.AbstractParameterIndex: ...
    def get_value(self) -> pydrake.systems.framework.AbstractParameterIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.AbstractParameterIndex) -> None: ...

class Value𝓣AbstractStateIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.AbstractStateIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.AbstractStateIndex: ...
    def get_value(self) -> pydrake.systems.framework.AbstractStateIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.AbstractStateIndex) -> None: ...

class Value𝓣BasicVector_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.BasicVector_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.BasicVector_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.systems.framework.BasicVector_TAutoDiffXdU: ...
    @overload
    def set_value(self, arg0: pydrake.systems.framework.BasicVector_TAutoDiffXdU) -> None: ...
    @overload
    def set_value(self, value: pydrake.autodiffutils.AutoDiffXd) -> None: ...
    @overload
    def set_value(self, value: numpy.ndarray[object[m, 1]]) -> None: ...

class Value𝓣BasicVector_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.BasicVector_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.BasicVector_TExpressionU: ...
    def get_value(self) -> pydrake.systems.framework.BasicVector_TExpressionU: ...
    @overload
    def set_value(self, arg0: pydrake.systems.framework.BasicVector_TExpressionU) -> None: ...
    @overload
    def set_value(self, value: pydrake.symbolic.Expression) -> None: ...
    @overload
    def set_value(self, value: numpy.ndarray[object[m, 1]]) -> None: ...

class Value𝓣BasicVector𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.BasicVector) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.BasicVector: ...
    def get_value(self) -> pydrake.systems.framework.BasicVector: ...
    @overload
    def set_value(self, arg0: pydrake.systems.framework.BasicVector) -> None: ...
    @overload
    def set_value(self, value: float) -> None: ...
    @overload
    def set_value(self, value: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...

class Value𝓣BodyIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.tree.BodyIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_value(self) -> pydrake.multibody.tree.BodyIndex: ...
    def set_value(self, arg0: pydrake.multibody.tree.BodyIndex) -> None: ...

class Value𝓣BusValue𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.BusValue) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.BusValue: ...
    def get_value(self) -> pydrake.systems.framework.BusValue: ...
    def set_value(self, arg0: pydrake.systems.framework.BusValue) -> None: ...

class Value𝓣CSpaceSeparatingPlane_𝓣Variable𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.optimization.CSpaceSeparatingPlane_TVariableU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.optimization.CSpaceSeparatingPlane_TVariableU: ...
    def get_value(self) -> pydrake.geometry.optimization.CSpaceSeparatingPlane_TVariableU: ...
    def set_value(self, arg0: pydrake.geometry.optimization.CSpaceSeparatingPlane_TVariableU) -> None: ...

class Value𝓣CSpaceSeparatingPlane𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.optimization.CSpaceSeparatingPlane) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.optimization.CSpaceSeparatingPlane: ...
    def get_value(self) -> pydrake.geometry.optimization.CSpaceSeparatingPlane: ...
    def set_value(self, arg0: pydrake.geometry.optimization.CSpaceSeparatingPlane) -> None: ...

class Value𝓣CacheIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.CacheIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.CacheIndex: ...
    def get_value(self) -> pydrake.systems.framework.CacheIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.CacheIndex) -> None: ...

class Value𝓣ClarabelSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.ClarabelSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.ClarabelSolverDetails: ...
    def get_value(self) -> pydrake.solvers.ClarabelSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.ClarabelSolverDetails) -> None: ...

class Value𝓣ClpSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.ClpSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.ClpSolverDetails: ...
    def get_value(self) -> pydrake.solvers.ClpSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.ClpSolverDetails) -> None: ...

class Value𝓣ContactResults_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.ContactResults_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.ContactResults_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.plant.ContactResults_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.plant.ContactResults_TAutoDiffXdU) -> None: ...

class Value𝓣ContactResults_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.ContactResults_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.ContactResults_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.plant.ContactResults_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.plant.ContactResults_TExpressionU) -> None: ...

class Value𝓣ContactResults𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.ContactResults) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.ContactResults: ...
    def get_value(self) -> pydrake.multibody.plant.ContactResults: ...
    def set_value(self, arg0: pydrake.multibody.plant.ContactResults) -> None: ...

class Value𝓣ContactWrench𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.optimization.ContactWrench) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.optimization.ContactWrench: ...
    def get_value(self) -> pydrake.multibody.optimization.ContactWrench: ...
    def set_value(self, arg0: pydrake.multibody.optimization.ContactWrench) -> None: ...

class Value𝓣ContinuousStateIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.ContinuousStateIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.ContinuousStateIndex: ...
    def get_value(self) -> pydrake.systems.framework.ContinuousStateIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.ContinuousStateIndex) -> None: ...

class Value𝓣CoulombFriction_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.CoulombFriction_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.CoulombFriction_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.plant.CoulombFriction_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.plant.CoulombFriction_TAutoDiffXdU) -> None: ...

class Value𝓣CoulombFriction_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.CoulombFriction_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.CoulombFriction_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.plant.CoulombFriction_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.plant.CoulombFriction_TExpressionU) -> None: ...

class Value𝓣CoulombFriction𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.CoulombFriction) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.CoulombFriction: ...
    def get_value(self) -> pydrake.multibody.plant.CoulombFriction: ...
    def set_value(self, arg0: pydrake.multibody.plant.CoulombFriction) -> None: ...

class Value𝓣CsdpSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.CsdpSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.CsdpSolverDetails: ...
    def get_value(self) -> pydrake.solvers.CsdpSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.CsdpSolverDetails) -> None: ...

class Value𝓣DeformableBodyIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.tree.DeformableBodyIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.tree.DeformableBodyIndex: ...
    def get_value(self) -> pydrake.multibody.tree.DeformableBodyIndex: ...
    def set_value(self, arg0: pydrake.multibody.tree.DeformableBodyIndex) -> None: ...

class Value𝓣DeformableContactInfo_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.DeformableContactInfo_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.DeformableContactInfo_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.plant.DeformableContactInfo_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.plant.DeformableContactInfo_TAutoDiffXdU) -> None: ...

class Value𝓣DeformableContactInfo_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.DeformableContactInfo_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.DeformableContactInfo_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.plant.DeformableContactInfo_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.plant.DeformableContactInfo_TExpressionU) -> None: ...

class Value𝓣DeformableContactInfo𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.DeformableContactInfo) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.DeformableContactInfo: ...
    def get_value(self) -> pydrake.multibody.plant.DeformableContactInfo: ...
    def set_value(self, arg0: pydrake.multibody.plant.DeformableContactInfo) -> None: ...

class Value𝓣DependencyTicket𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.DependencyTicket) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.DependencyTicket: ...
    def get_value(self) -> pydrake.systems.framework.DependencyTicket: ...
    def set_value(self, arg0: pydrake.systems.framework.DependencyTicket) -> None: ...

class Value𝓣DiscreteStateIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.DiscreteStateIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.DiscreteStateIndex: ...
    def get_value(self) -> pydrake.systems.framework.DiscreteStateIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.DiscreteStateIndex) -> None: ...

class Value𝓣ExternallyAppliedSpatialForce_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.ExternallyAppliedSpatialForce_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.ExternallyAppliedSpatialForce_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.plant.ExternallyAppliedSpatialForce_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.plant.ExternallyAppliedSpatialForce_TAutoDiffXdU) -> None: ...

class Value𝓣ExternallyAppliedSpatialForce_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.ExternallyAppliedSpatialForce_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.ExternallyAppliedSpatialForce_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.plant.ExternallyAppliedSpatialForce_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.plant.ExternallyAppliedSpatialForce_TExpressionU) -> None: ...

class Value𝓣ExternallyAppliedSpatialForce𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.ExternallyAppliedSpatialForce) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.ExternallyAppliedSpatialForce: ...
    def get_value(self) -> pydrake.multibody.plant.ExternallyAppliedSpatialForce: ...
    def set_value(self, arg0: pydrake.multibody.plant.ExternallyAppliedSpatialForce) -> None: ...

class Value𝓣ForceElementIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.tree.ForceElementIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.tree.ForceElementIndex: ...
    def get_value(self) -> pydrake.multibody.tree.ForceElementIndex: ...
    def set_value(self, arg0: pydrake.multibody.tree.ForceElementIndex) -> None: ...

class Value𝓣FrameIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.tree.FrameIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.tree.FrameIndex: ...
    def get_value(self) -> pydrake.multibody.tree.FrameIndex: ...
    def set_value(self, arg0: pydrake.multibody.tree.FrameIndex) -> None: ...

class Value𝓣FramePoseVector_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.FramePoseVector_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.FramePoseVector_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.geometry.FramePoseVector_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.geometry.FramePoseVector_TAutoDiffXdU) -> None: ...

class Value𝓣FramePoseVector_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.FramePoseVector_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.FramePoseVector_TExpressionU: ...
    def get_value(self) -> pydrake.geometry.FramePoseVector_TExpressionU: ...
    def set_value(self, arg0: pydrake.geometry.FramePoseVector_TExpressionU) -> None: ...

class Value𝓣FramePoseVector𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.FramePoseVector) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.FramePoseVector: ...
    def get_value(self) -> pydrake.geometry.FramePoseVector: ...
    def set_value(self, arg0: pydrake.geometry.FramePoseVector) -> None: ...

class Value𝓣GeometryConfigurationVector_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.GeometryConfigurationVector_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.GeometryConfigurationVector_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.geometry.GeometryConfigurationVector_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.geometry.GeometryConfigurationVector_TAutoDiffXdU) -> None: ...

class Value𝓣GeometryConfigurationVector_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.GeometryConfigurationVector_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.GeometryConfigurationVector_TExpressionU: ...
    def get_value(self) -> pydrake.geometry.GeometryConfigurationVector_TExpressionU: ...
    def set_value(self, arg0: pydrake.geometry.GeometryConfigurationVector_TExpressionU) -> None: ...

class Value𝓣GeometryConfigurationVector𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.GeometryConfigurationVector) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.GeometryConfigurationVector: ...
    def get_value(self) -> pydrake.geometry.GeometryConfigurationVector: ...
    def set_value(self, arg0: pydrake.geometry.GeometryConfigurationVector) -> None: ...

class Value𝓣GurobiSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.GurobiSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.GurobiSolverDetails: ...
    def get_value(self) -> pydrake.solvers.GurobiSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.GurobiSolverDetails) -> None: ...

class Value𝓣HydroelasticContactInfo_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.HydroelasticContactInfo_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.HydroelasticContactInfo_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.plant.HydroelasticContactInfo_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.plant.HydroelasticContactInfo_TAutoDiffXdU) -> None: ...

class Value𝓣HydroelasticContactInfo_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.HydroelasticContactInfo_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.HydroelasticContactInfo_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.plant.HydroelasticContactInfo_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.plant.HydroelasticContactInfo_TExpressionU) -> None: ...

class Value𝓣HydroelasticContactInfo𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.plant.HydroelasticContactInfo) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.plant.HydroelasticContactInfo: ...
    def get_value(self) -> pydrake.multibody.plant.HydroelasticContactInfo: ...
    def set_value(self, arg0: pydrake.multibody.plant.HydroelasticContactInfo) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kBgr8U𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkBgr8UU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkBgr8UU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkBgr8UU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkBgr8UU) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kBgra8U𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkBgra8UU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkBgra8UU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkBgra8UU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkBgra8UU) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kDepth16U𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkDepth16UU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkDepth16UU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkDepth16UU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkDepth16UU) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kDepth32F𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkDepth32FU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkDepth32FU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkDepth32FU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkDepth32FU) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kGrey8U𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkGrey8UU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkGrey8UU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkGrey8UU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkGrey8UU) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kLabel16I𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkLabel16IU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkLabel16IU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkLabel16IU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkLabel16IU) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kRgb8U𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkRgb8UU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkRgb8UU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkRgb8UU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkRgb8UU) -> None: ...

class Value𝓣Image𝓣PixelType𝓹kRgba8U𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkRgba8UU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkRgba8UU: ...
    def get_value(self) -> pydrake.systems.sensors.ImageTPixelTypepkRgba8UU: ...
    def set_value(self, arg0: pydrake.systems.sensors.ImageTPixelTypepkRgba8UU) -> None: ...

class Value𝓣InputPortIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.InputPortIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.InputPortIndex: ...
    def get_value(self) -> pydrake.systems.framework.InputPortIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.InputPortIndex) -> None: ...

class Value𝓣IpoptSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.IpoptSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.IpoptSolverDetails: ...
    def get_value(self) -> pydrake.solvers.IpoptSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.IpoptSolverDetails) -> None: ...

class Value𝓣Isometry3_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.common.eigen_geometry.Isometry3_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.common.eigen_geometry.Isometry3_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.common.eigen_geometry.Isometry3_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.common.eigen_geometry.Isometry3_TAutoDiffXdU) -> None: ...

class Value𝓣Isometry3_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.common.eigen_geometry.Isometry3_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.common.eigen_geometry.Isometry3_TExpressionU: ...
    def get_value(self) -> pydrake.common.eigen_geometry.Isometry3_TExpressionU: ...
    def set_value(self, arg0: pydrake.common.eigen_geometry.Isometry3_TExpressionU) -> None: ...

class Value𝓣Isometry3𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.common.eigen_geometry.Isometry3) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.common.eigen_geometry.Isometry3: ...
    def get_value(self) -> pydrake.common.eigen_geometry.Isometry3: ...
    def set_value(self, arg0: pydrake.common.eigen_geometry.Isometry3) -> None: ...

class Value𝓣JointActuatorIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.tree.JointActuatorIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.tree.JointActuatorIndex: ...
    def get_value(self) -> pydrake.multibody.tree.JointActuatorIndex: ...
    def set_value(self, arg0: pydrake.multibody.tree.JointActuatorIndex) -> None: ...

class Value𝓣JointIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.tree.JointIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.tree.JointIndex: ...
    def get_value(self) -> pydrake.multibody.tree.JointIndex: ...
    def set_value(self, arg0: pydrake.multibody.tree.JointIndex) -> None: ...

class Value𝓣ModelInstanceIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.tree.ModelInstanceIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.tree.ModelInstanceIndex: ...
    def get_value(self) -> pydrake.multibody.tree.ModelInstanceIndex: ...
    def set_value(self, arg0: pydrake.multibody.tree.ModelInstanceIndex) -> None: ...

class Value𝓣MosekSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.MosekSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.MosekSolverDetails: ...
    def get_value(self) -> pydrake.solvers.MosekSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.MosekSolverDetails) -> None: ...

class Value𝓣NloptSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.NloptSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.NloptSolverDetails: ...
    def get_value(self) -> pydrake.solvers.NloptSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.NloptSolverDetails) -> None: ...

class Value𝓣NumericParameterIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.NumericParameterIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.NumericParameterIndex: ...
    def get_value(self) -> pydrake.systems.framework.NumericParameterIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.NumericParameterIndex) -> None: ...

class Value𝓣OsqpSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.OsqpSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.OsqpSolverDetails: ...
    def get_value(self) -> pydrake.solvers.OsqpSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.OsqpSolverDetails) -> None: ...

class Value𝓣OutputPortIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.OutputPortIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.OutputPortIndex: ...
    def get_value(self) -> pydrake.systems.framework.OutputPortIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.OutputPortIndex) -> None: ...

class Value𝓣PointCloud𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.perception.PointCloud) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.perception.PointCloud: ...
    def get_value(self) -> pydrake.perception.PointCloud: ...
    def set_value(self, arg0: pydrake.perception.PointCloud) -> None: ...

class Value𝓣QueryObject_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.QueryObject_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.QueryObject_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.geometry.QueryObject_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.geometry.QueryObject_TAutoDiffXdU) -> None: ...

class Value𝓣QueryObject_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.QueryObject_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.QueryObject_TExpressionU: ...
    def get_value(self) -> pydrake.geometry.QueryObject_TExpressionU: ...
    def set_value(self, arg0: pydrake.geometry.QueryObject_TExpressionU) -> None: ...

class Value𝓣QueryObject𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.QueryObject) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.QueryObject: ...
    def get_value(self) -> pydrake.geometry.QueryObject: ...
    def set_value(self, arg0: pydrake.geometry.QueryObject) -> None: ...

class Value𝓣RenderLabel𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.RenderLabel) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.RenderLabel: ...
    def get_value(self) -> pydrake.geometry.RenderLabel: ...
    def set_value(self, arg0: pydrake.geometry.RenderLabel) -> None: ...

class Value𝓣Rgba𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.geometry.Rgba) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.geometry.Rgba: ...
    def get_value(self) -> pydrake.geometry.Rgba: ...
    def set_value(self, arg0: pydrake.geometry.Rgba) -> None: ...

class Value𝓣RigidTransform_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.math.RigidTransform_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.math.RigidTransform_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.math.RigidTransform_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.math.RigidTransform_TAutoDiffXdU) -> None: ...

class Value𝓣RigidTransform_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.math.RigidTransform_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.math.RigidTransform_TExpressionU: ...
    def get_value(self) -> pydrake.math.RigidTransform_TExpressionU: ...
    def set_value(self, arg0: pydrake.math.RigidTransform_TExpressionU) -> None: ...

class Value𝓣RigidTransform𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.math.RigidTransform) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.math.RigidTransform: ...
    def get_value(self) -> pydrake.math.RigidTransform: ...
    def set_value(self, arg0: pydrake.math.RigidTransform) -> None: ...

class Value𝓣RotationMatrix_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.math.RotationMatrix_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.math.RotationMatrix_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.math.RotationMatrix_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.math.RotationMatrix_TAutoDiffXdU) -> None: ...

class Value𝓣RotationMatrix_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.math.RotationMatrix_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.math.RotationMatrix_TExpressionU: ...
    def get_value(self) -> pydrake.math.RotationMatrix_TExpressionU: ...
    def set_value(self, arg0: pydrake.math.RotationMatrix_TExpressionU) -> None: ...

class Value𝓣RotationMatrix𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.math.RotationMatrix) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.math.RotationMatrix: ...
    def get_value(self) -> pydrake.math.RotationMatrix: ...
    def set_value(self, arg0: pydrake.math.RotationMatrix) -> None: ...

class Value𝓣ScsSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.ScsSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.ScsSolverDetails: ...
    def get_value(self) -> pydrake.solvers.ScsSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.ScsSolverDetails) -> None: ...

class Value𝓣SnoptSolverDetails𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.solvers.SnoptSolverDetails) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.solvers.SnoptSolverDetails: ...
    def get_value(self) -> pydrake.solvers.SnoptSolverDetails: ...
    def set_value(self, arg0: pydrake.solvers.SnoptSolverDetails) -> None: ...

class Value𝓣SpatialAcceleration_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialAcceleration_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialAcceleration_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialAcceleration_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialAcceleration_TAutoDiffXdU) -> None: ...

class Value𝓣SpatialAcceleration_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialAcceleration_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialAcceleration_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialAcceleration_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialAcceleration_TExpressionU) -> None: ...

class Value𝓣SpatialAcceleration𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialAcceleration) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialAcceleration: ...
    def get_value(self) -> pydrake.multibody.math.SpatialAcceleration: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialAcceleration) -> None: ...

class Value𝓣SpatialForce_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialForce_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialForce_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialForce_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialForce_TAutoDiffXdU) -> None: ...

class Value𝓣SpatialForce_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialForce_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialForce_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialForce_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialForce_TExpressionU) -> None: ...

class Value𝓣SpatialForce𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialForce) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialForce: ...
    def get_value(self) -> pydrake.multibody.math.SpatialForce: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialForce) -> None: ...

class Value𝓣SpatialMomentum_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialMomentum_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialMomentum_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialMomentum_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialMomentum_TAutoDiffXdU) -> None: ...

class Value𝓣SpatialMomentum_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialMomentum_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialMomentum_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialMomentum_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialMomentum_TExpressionU) -> None: ...

class Value𝓣SpatialMomentum𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialMomentum) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialMomentum: ...
    def get_value(self) -> pydrake.multibody.math.SpatialMomentum: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialMomentum) -> None: ...

class Value𝓣SpatialVelocity_𝓣AutoDiffXd𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialVelocity_TAutoDiffXdU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialVelocity_TAutoDiffXdU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialVelocity_TAutoDiffXdU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialVelocity_TAutoDiffXdU) -> None: ...

class Value𝓣SpatialVelocity_𝓣Expression𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialVelocity_TExpressionU) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialVelocity_TExpressionU: ...
    def get_value(self) -> pydrake.multibody.math.SpatialVelocity_TExpressionU: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialVelocity_TExpressionU) -> None: ...

class Value𝓣SpatialVelocity𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.multibody.math.SpatialVelocity) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.multibody.math.SpatialVelocity: ...
    def get_value(self) -> pydrake.multibody.math.SpatialVelocity: ...
    def set_value(self, arg0: pydrake.multibody.math.SpatialVelocity) -> None: ...

class Value𝓣SubsystemIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.SubsystemIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.SubsystemIndex: ...
    def get_value(self) -> pydrake.systems.framework.SubsystemIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.SubsystemIndex) -> None: ...

class Value𝓣SystemConstraintIndex𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: pydrake.systems.framework.SystemConstraintIndex) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> pydrake.systems.framework.SystemConstraintIndex: ...
    def get_value(self) -> pydrake.systems.framework.SystemConstraintIndex: ...
    def set_value(self, arg0: pydrake.systems.framework.SystemConstraintIndex) -> None: ...

class Value𝓣bool𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: bool) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> bool: ...
    def set_value(self, arg0: bool) -> None: ...

class Value𝓣float𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: float) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> float: ...
    def set_value(self, arg0: float) -> None: ...

class Value𝓣list𝓣ExternallyAppliedSpatialForce_𝓣AutoDiffXd𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.plant.ExternallyAppliedSpatialForce_TAutoDiffXdU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.plant.ExternallyAppliedSpatialForce_TAutoDiffXdU]: ...
    def set_value(self, arg0: list[pydrake.multibody.plant.ExternallyAppliedSpatialForce_TAutoDiffXdU]) -> None: ...

class Value𝓣list𝓣ExternallyAppliedSpatialForce_𝓣Expression𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.plant.ExternallyAppliedSpatialForce_TExpressionU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.plant.ExternallyAppliedSpatialForce_TExpressionU]: ...
    def set_value(self, arg0: list[pydrake.multibody.plant.ExternallyAppliedSpatialForce_TExpressionU]) -> None: ...

class Value𝓣list𝓣ExternallyAppliedSpatialForce𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.plant.ExternallyAppliedSpatialForce]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.plant.ExternallyAppliedSpatialForce]: ...
    def set_value(self, arg0: list[pydrake.multibody.plant.ExternallyAppliedSpatialForce]) -> None: ...

class Value𝓣list𝓣RigidTransform_𝓣AutoDiffXd𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.math.RigidTransform_TAutoDiffXdU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.math.RigidTransform_TAutoDiffXdU]: ...
    def set_value(self, arg0: list[pydrake.math.RigidTransform_TAutoDiffXdU]) -> None: ...

class Value𝓣list𝓣RigidTransform_𝓣Expression𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.math.RigidTransform_TExpressionU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.math.RigidTransform_TExpressionU]: ...
    def set_value(self, arg0: list[pydrake.math.RigidTransform_TExpressionU]) -> None: ...

class Value𝓣list𝓣RigidTransform𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.math.RigidTransform]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.math.RigidTransform]: ...
    def set_value(self, arg0: list[pydrake.math.RigidTransform]) -> None: ...

class Value𝓣list𝓣RotationMatrix_𝓣AutoDiffXd𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.math.RotationMatrix_TAutoDiffXdU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.math.RotationMatrix_TAutoDiffXdU]: ...
    def set_value(self, arg0: list[pydrake.math.RotationMatrix_TAutoDiffXdU]) -> None: ...

class Value𝓣list𝓣RotationMatrix_𝓣Expression𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.math.RotationMatrix_TExpressionU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.math.RotationMatrix_TExpressionU]: ...
    def set_value(self, arg0: list[pydrake.math.RotationMatrix_TExpressionU]) -> None: ...

class Value𝓣list𝓣RotationMatrix𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.math.RotationMatrix]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.math.RotationMatrix]: ...
    def set_value(self, arg0: list[pydrake.math.RotationMatrix]) -> None: ...

class Value𝓣list𝓣SpatialAcceleration_𝓣AutoDiffXd𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialAcceleration_TAutoDiffXdU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialAcceleration_TAutoDiffXdU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialAcceleration_TAutoDiffXdU]) -> None: ...

class Value𝓣list𝓣SpatialAcceleration_𝓣Expression𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialAcceleration_TExpressionU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialAcceleration_TExpressionU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialAcceleration_TExpressionU]) -> None: ...

class Value𝓣list𝓣SpatialAcceleration𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialAcceleration]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialAcceleration]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialAcceleration]) -> None: ...

class Value𝓣list𝓣SpatialForce_𝓣AutoDiffXd𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialForce_TAutoDiffXdU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialForce_TAutoDiffXdU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialForce_TAutoDiffXdU]) -> None: ...

class Value𝓣list𝓣SpatialForce_𝓣Expression𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialForce_TExpressionU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialForce_TExpressionU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialForce_TExpressionU]) -> None: ...

class Value𝓣list𝓣SpatialForce𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialForce]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialForce]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialForce]) -> None: ...

class Value𝓣list𝓣SpatialMomentum_𝓣AutoDiffXd𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialMomentum_TAutoDiffXdU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialMomentum_TAutoDiffXdU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialMomentum_TAutoDiffXdU]) -> None: ...

class Value𝓣list𝓣SpatialMomentum_𝓣Expression𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialMomentum_TExpressionU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialMomentum_TExpressionU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialMomentum_TExpressionU]) -> None: ...

class Value𝓣list𝓣SpatialMomentum𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialMomentum]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialMomentum]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialMomentum]) -> None: ...

class Value𝓣list𝓣SpatialVelocity_𝓣AutoDiffXd𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialVelocity_TAutoDiffXdU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialVelocity_TAutoDiffXdU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialVelocity_TAutoDiffXdU]) -> None: ...

class Value𝓣list𝓣SpatialVelocity_𝓣Expression𝓤𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialVelocity_TExpressionU]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialVelocity_TExpressionU]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialVelocity_TExpressionU]) -> None: ...

class Value𝓣list𝓣SpatialVelocity𝓤𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: list[pydrake.multibody.math.SpatialVelocity]) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> list[pydrake.multibody.math.SpatialVelocity]: ...
    def set_value(self, arg0: list[pydrake.multibody.math.SpatialVelocity]) -> None: ...

class Value𝓣object𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: object) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> object: ...
    def get_value(self) -> object: ...
    def set_value(self, arg0: object) -> None: ...

class Value𝓣str𝓤(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: str) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> str: ...
    def set_value(self, arg0: str) -> None: ...

class _TemporaryName_N5drake5ValueINSt7__cxx1112basic_stringIcSt11char_traitsIcESaIcEEEEE(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: str) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> str: ...
    def set_value(self, arg0: str) -> None: ...

class _TemporaryName_N5drake5ValueIbEE(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: bool) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> bool: ...
    def set_value(self, arg0: bool) -> None: ...

class _TemporaryName_N5drake5ValueIdEE(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: float) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> None: ...
    def get_value(self) -> float: ...
    def set_value(self, arg0: float) -> None: ...

class _TemporaryName_N5drake7pydrake8internal12_GLOBAL__N_113PyObjectValueE(AbstractValue):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, arg0: object) -> None: ...
    @overload
    def __init__(self, *args, **kwargs) -> Any: ...
    def get_mutable_value(self) -> object: ...
    def get_value(self) -> object: ...
    def set_value(self, arg0: object) -> None: ...
